"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = require("fs-extra");
const os_1 = require("os");
const oni_types_1 = require("oni-types");
const path = require("path");
const QuickOpenItem_1 = require("./QuickOpenItem");
class BookmarkSearch {
    constructor(_oni) {
        this._oni = _oni;
        this._onSearchResults = new oni_types_1.Event();
        this._getBookmarkPath = (bookmark) => {
            if (path.isAbsolute(bookmark)) {
                return bookmark;
            }
            const userBaseDir = this._oni.configuration.getValue("oni.bookmarks.baseDirectory");
            const baseDir = userBaseDir || os_1.homedir();
            return path.join(baseDir, bookmark);
        };
        this.handleBookmark = ({ metadata }) => __awaiter(this, void 0, void 0, function* () {
            const bookmarkPath = this._getBookmarkPath(metadata.path);
            yield this._handleItem(bookmarkPath);
        });
        // deliberately throw errors as a means of clarifying if a path exists
        // i.e. if isDir => true, else => false, if does not exist => new Error(err)
        this.isDir = (path) => __awaiter(this, void 0, void 0, function* () {
            const stats = yield fs_extra_1.stat(path);
            return stats.isDirectory();
        });
        this._handleItem = (bookmarkPath) => __awaiter(this, void 0, void 0, function* () {
            try {
                const isDirectory = yield this.isDir(bookmarkPath);
                return isDirectory
                    ? this._oni.workspace.changeDirectory(bookmarkPath)
                    : this._oni.editors.openFile(bookmarkPath);
            }
            catch (error) {
                this._oni.log.warn(`[Oni Bookmarks Menu Error]: The Bookmark path ${bookmarkPath} does not exist: \n ${error.message}`);
            }
        });
        const bookmarks = this._oni.configuration.getValue("oni.bookmarks", []);
        // TODO: Consider adding folders as well (recursive async with ignores/excludes)
        this._bookmarkItems = [
            new QuickOpenItem_1.QuickOpenItem("Open Folder", "", QuickOpenItem_1.QuickOpenType.folderHelp),
            ...bookmarks.map(folder => new QuickOpenItem_1.QuickOpenItem(folder, "", QuickOpenItem_1.QuickOpenType.bookmark, folder)),
            new QuickOpenItem_1.QuickOpenItem("Open configuration", "For adding a bookmark", QuickOpenItem_1.QuickOpenType.bookmarkHelp),
        ];
    }
    cancel() { }
    changeQueryText(newText) {
        this._onSearchResults.dispatch({
            items: this._bookmarkItems,
            isComplete: true,
        });
    }
    get onSearchResults() {
        return this._onSearchResults;
    }
}
exports.default = BookmarkSearch;
//# sourceMappingURL=BookmarksSearch.js.map