"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var QuickOpenType;
(function (QuickOpenType) {
    QuickOpenType[QuickOpenType["bookmark"] = 0] = "bookmark";
    QuickOpenType[QuickOpenType["bookmarkHelp"] = 1] = "bookmarkHelp";
    QuickOpenType[QuickOpenType["file"] = 2] = "file";
    QuickOpenType[QuickOpenType["folder"] = 3] = "folder";
    QuickOpenType[QuickOpenType["folderHelp"] = 4] = "folderHelp";
    QuickOpenType[QuickOpenType["bufferLine"] = 5] = "bufferLine";
})(QuickOpenType = exports.QuickOpenType || (exports.QuickOpenType = {}));
function getTypeFromMenuItem(item) {
    return QuickOpenType[item.metadata["qo-type"]];
}
exports.getTypeFromMenuItem = getTypeFromMenuItem;
// Wrapper around quick open items, this not only allows us to show multiple icons
// It also allows us to distinguish between types of items
class QuickOpenItem {
    constructor(_label, _detail, _type, _path = "", _line = 0, _column = 0) {
        this._label = _label;
        this._detail = _detail;
        this._type = _type;
        this._path = _path;
        this._line = _line;
        this._column = _column;
        this._label = _label.trim();
        this._detail = _detail.trim();
    }
    toMenuItem(oni, pinned) {
        return {
            icon: this.getIcon(oni),
            label: this._label,
            detail: this._detail,
            metadata: {
                "qo-type": QuickOpenType[this._type],
                hash: this.hash,
                path: this._path,
                line: this._line.toString(),
                column: this._column.toString(),
            },
            pinned,
        };
    }
    toQuickFixItem() {
        return {
            filename: this._path,
            lnum: this._line,
            col: this._column,
            text: (this._label + " " + this._detail).trim(),
        };
    }
    get hash() {
        return [
            this._label,
            this._detail,
            QuickOpenType[this._type],
            this._path,
            String(this._line),
            String(this._column),
        ].join("|");
    }
    getIcon(oni) {
        // Return a fa icon by type
        switch (this._type) {
            case QuickOpenType.bookmark:
                return "star-o";
            case QuickOpenType.bookmarkHelp:
                return "info";
            case QuickOpenType.file:
                return oni.ui.getFileIcon(this._label);
            case QuickOpenType.folder:
                return "folder-o";
            case QuickOpenType.folderHelp:
                return "folder-open-o";
            case QuickOpenType.bufferLine:
                return "angle-right";
            default:
                return "question-circle-o";
        }
    }
}
exports.QuickOpenItem = QuickOpenItem;
//# sourceMappingURL=QuickOpenItem.js.map