"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const cli_args_1 = require("./cli_args");
const child_process_1 = require("child_process");
function main(cli_arguments) {
    return __awaiter(this, void 0, void 0, function* () {
        let args;
        try {
            args = cli_args_1.parseCLIProcessArgv(cli_arguments);
        }
        catch (err) {
            process.stdout.write(err.message);
            throw err;
        }
        if (args.help || args.version) {
            const version = require(path.join(__dirname, "..", "..", "..", "package.json")).version;
            process.stdout.write("Oni: Modern Modal Editing - powered by Neovim\n");
            process.stdout.write(` version: ${version}\n`);
            process.stdout.write("\nUsage:\n oni [FILE]\t\tEdit file\n");
            process.stdout.write("\nhttps://github.com/onivim/oni\n");
            return Promise.resolve(true);
        }
        const shellEnvPromise = Promise.resolve().then(() => require("shell-env"));
        const shellEnv = (yield shellEnvPromise).sync();
        const env = assign({}, process.env, shellEnv, {
            ONI_CLI: "1",
            ELECTRON_NO_ATTACH_CONSOLE: "1",
        });
        delete env["ELECTRON_RUN_AS_NODE"];
        const options = {
            detached: true,
            env,
        };
        const child = yield child_process_1.spawn(process.execPath, cli_arguments.slice(2), options);
        child.unref();
        child.on("close", code => {
            if (code !== 0) {
                throw Error(`Exit code was ${code}, not 0.`);
            }
            else {
                return Promise.resolve(true);
            }
        });
        child.on("error", err => {
            throw err;
        });
        child.on("exit", code => {
            if (code && code !== 0) {
                throw Error(`Exit code was ${code}, not 0.`);
            }
            else {
                return Promise.resolve(true);
            }
        });
    });
}
exports.main = main;
function assign(destination, ...sources) {
    sources.forEach(source => Object.keys(source).forEach(key => (destination[key] = source[key])));
    return destination;
}
function eventuallyExit(code) {
    setTimeout(() => process.exit(code), 0);
}
main(process.argv)
    .then(() => {
    eventuallyExit(0);
})
    .then(null, err => {
    console.error(err.message || err.stack || err);
    eventuallyExit(1);
});
//# sourceMappingURL=cli.js.map