"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Log = require("./Log");
function moveToNextOniInstance(windows, direction) {
    const currentFocusedWindows = windows.filter(f => f.isFocused());
    if (currentFocusedWindows.length === 0) {
        Log.info("No window currently focused");
        return;
    }
    else if (windows.length === 1) {
        Log.info("No window to swap to");
        return;
    }
    const currentFocusedWindow = currentFocusedWindows[0];
    const windowsToCheck = windows.filter(window => window !== currentFocusedWindow && !window.isMinimized());
    const validWindows = windowsToCheck.filter(window => windowIsInValidDirection(direction, currentFocusedWindow.getBounds(), window.getBounds()));
    if (validWindows.length === 0) {
        return;
    }
    const windowToSwapTo = validWindows.reduce((curr, prev) => {
        const isCurrentWindowBetter = checkWindowToFindBest(currentFocusedWindow.getBounds(), curr.getBounds(), prev.getBounds(), direction);
        if (isCurrentWindowBetter) {
            return curr;
        }
        else {
            return prev;
        }
    }, validWindows[0]);
    windows[windows.indexOf(windowToSwapTo)].focus();
}
exports.moveToNextOniInstance = moveToNextOniInstance;
function windowIsInValidDirection(direction, currentPos, testPos) {
    let valuesIncrease = false;
    let coord = "x";
    switch (direction) {
        case "left":
            valuesIncrease = false;
            break;
        case "right":
            valuesIncrease = true;
            break;
        case "up":
            valuesIncrease = false;
            coord = "y";
            break;
        case "down":
            valuesIncrease = true;
            coord = "y";
            break;
        default:
            return false;
    }
    if (valuesIncrease) {
        if (testPos[coord] > currentPos[coord]) {
            return true;
        }
    }
    else {
        if (testPos[coord] < currentPos[coord]) {
            return true;
        }
    }
    return false;
}
exports.windowIsInValidDirection = windowIsInValidDirection;
function checkWindowToFindBest(currentWindow, testWindow, currentBest, direction) {
    const differenceInX = Math.abs(currentWindow.x - testWindow.x);
    const differenceInY = Math.abs(currentWindow.y - testWindow.y);
    const bestDiffInX = Math.abs(currentWindow.x - currentBest.x);
    const bestDiffInY = Math.abs(currentWindow.y - currentBest.y);
    let mainAxisDiff = DistanceComparison.larger;
    let secondAxisDiff = DistanceComparison.larger;
    switch (direction) {
        case "left":
        case "right":
            mainAxisDiff = compareDistances(differenceInX, bestDiffInX);
            secondAxisDiff = compareDistances(differenceInY, bestDiffInY);
            break;
        case "up":
        case "down":
            mainAxisDiff = compareDistances(differenceInY, bestDiffInY);
            secondAxisDiff = compareDistances(differenceInX, bestDiffInX);
            break;
        default:
            return false;
    }
    if (mainAxisDiff === DistanceComparison.smaller) {
        return true;
    }
    else if (mainAxisDiff === DistanceComparison.equal &&
        secondAxisDiff === DistanceComparison.smaller) {
        return true;
    }
    else {
        return false;
    }
}
exports.checkWindowToFindBest = checkWindowToFindBest;
var DistanceComparison;
(function (DistanceComparison) {
    DistanceComparison[DistanceComparison["smaller"] = 0] = "smaller";
    DistanceComparison[DistanceComparison["larger"] = 1] = "larger";
    DistanceComparison[DistanceComparison["equal"] = 2] = "equal";
})(DistanceComparison = exports.DistanceComparison || (exports.DistanceComparison = {}));
function compareDistances(currentDifference, bestDifference) {
    if (currentDifference === bestDifference) {
        return DistanceComparison.equal;
    }
    else if (currentDifference < bestDifference) {
        return DistanceComparison.smaller;
    }
    else {
        return DistanceComparison.larger;
    }
}
exports.compareDistances = compareDistances;
//# sourceMappingURL=WindowManager.js.map