"use strict";
/**
 * Entry point for 'sneak integration' + Oni augmentations in the browser
 *
 * This script is a 'preload' script for the webview. You can see more info here:
 * https://electronjs.org/docs/api/webview-tag#preload
 */
(function () {
    var __oni_win = window;
    var ipcRenderer = require("electron").ipcRenderer;
    window.document.addEventListener("focusin", function (evt) {
        var target = evt.target;
        ipcRenderer.sendToHost("focusin", target ? target.tagName : null);
    });
    window.document.addEventListener("focusout", function (evt) {
        ipcRenderer.sendToHost("focusout");
    });
    var createRectangle = function (x, y, width, height) { return ({
        x: x,
        y: y,
        width: width,
        height: height,
    }); };
    var _tags = [];
    // Note that this name must be in sync with 'BrowserView.tsx'
    __oni_win["__oni_sneak_collector__"] = function () {
        _tags = [];
        var idx = 0;
        var width = window.innerWidth;
        var height = window.innerHeight;
        var addElement = function (element) {
            idx++;
            var clientRect = element.getBoundingClientRect();
            if (clientRect.width === 0 || clientRect.height === 0) {
                return;
            }
            var isInBounds = clientRect.top >= 0 &&
                clientRect.left >= 0 &&
                clientRect.top <= height &&
                clientRect.left <= width;
            if (!isInBounds) {
                return;
            }
            var callback = function (elem) { return function () {
                if (elem && elem.click) {
                    if (elem.tagName === "A") {
                        elem.click();
                    }
                    else if (elem.tagName === "INPUT" || elem.tagName === "TEXTAREA") {
                        elem.focus();
                    }
                }
            }; };
            _tags.push({
                rectangle: createRectangle(clientRect.left, clientRect.top, clientRect.width, clientRect.height),
                id: idx.toString(),
                callback: callback(element),
            });
        };
        var tagsToCollect = ["a", "input", "textarea"];
        tagsToCollect.forEach(function (tag) {
            var elems = document.getElementsByTagName(tag);
            for (var i = 0; i < elems.length; i++) {
                addElement(elems[i]);
            }
        });
        var ret = _tags.map(function (tag) { return ({
            id: tag.id,
            rectangle: tag.rectangle,
        }); });
        return ret;
    };
    __oni_win["__oni_sneak_execute__"] = function (id) {
        var matchingTags = _tags.filter(function (t) { return t.id === id; });
        if (matchingTags.length > 0) {
            var match = matchingTags[0];
            match.callback();
        }
    };
})();
