/**
 * LightweightLanguageClient.ts
 *
 * Helper class for an in-proc strategy for implementing a language client interface
 */
export declare type ServerRequestHandler = (requestName: string, payload: any) => Promise<any>;
export declare type ClientRequestHandler = (payload: any) => Promise<any>;
export declare type NotificationHandler = (notificationName: string, payload: any) => void;
import { Event } from "oni-types";
export declare class LanguageConnection {
    private _client;
    constructor(_client: LightweightLanguageClient);
    request(requestName: string, language: string, payload: any): Promise<any>;
    notify(notificationName: string, language: string, payload: any): void;
    subscribeToNotification(notificationName: string, notificationHandler: NotificationHandler): void;
    subscribeToRequest(requestName: string, handler: ServerRequestHandler): void;
}
export declare class LightweightLanguageClient {
    private _subscriptions;
    private _requestHandler;
    private _clientRequestHandler;
    private _notificationHandler;
    readonly connection: LanguageConnection;
    readonly serverCapabilities: any;
    subscribe(notificationName: string, evt: Event<any>): void;
    sendRequest<T>(fileName: string, requestName: string, protocolArguments: any): Promise<T>;
    sendNotification(fileName: string, notificationName: string, protocolArguments: any): Promise<void>;
    handleRequest(requestName: string, handler: ClientRequestHandler): void;
    _handleRequest(requestName: string, handler: ServerRequestHandler): void;
    _getClientRequestHandler(requestName: any): ClientRequestHandler;
    _handleNotification(notificationName: string, notificationHandler: NotificationHandler): void;
    _notify(notificationName: string, language: string, payload: any): void;
    private _unwrapThunk(valueOrThunk?);
}
