"use strict";
/**
 * LightweightLanguageClient.ts
 *
 * Helper class for an in-proc strategy for implementing a language client interface
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var LanguageConnection = /** @class */ (function () {
    function LanguageConnection(_client) {
        this._client = _client;
    }
    LanguageConnection.prototype.request = function (requestName, language, payload) {
        return this._client._getClientRequestHandler(requestName)({
            language: language,
            payload: payload,
        });
    };
    LanguageConnection.prototype.notify = function (notificationName, language, payload) {
        this._client._notify(notificationName, language, payload);
    };
    LanguageConnection.prototype.subscribeToNotification = function (notificationName, notificationHandler) {
        this._client._handleNotification(notificationName, notificationHandler);
    };
    LanguageConnection.prototype.subscribeToRequest = function (requestName, handler) {
        this._client._handleRequest(requestName, handler);
    };
    return LanguageConnection;
}());
exports.LanguageConnection = LanguageConnection;
var LightweightLanguageClient = /** @class */ (function () {
    function LightweightLanguageClient() {
        this._subscriptions = {};
        // This is confusing because the requests are handled both ways...
        // This dictionary tracks handlers on the 'server' side
        this._requestHandler = {};
        this._clientRequestHandler = {};
        this._notificationHandler = {};
    }
    Object.defineProperty(LightweightLanguageClient.prototype, "connection", {
        get: function () {
            return;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LightweightLanguageClient.prototype, "serverCapabilities", {
        get: function () {
            return {
                textDocumentSync: 2 /* incremental */,
                hoverProvider: true,
                completionProvider: {
                    resolveProvider: true,
                    triggerCharacters: ["."],
                },
                signatureHelpProvider: {
                    triggerCharacters: ["("],
                },
                definitionProvider: true,
                referencesProvider: true,
                formattingProvider: false,
                renameProvider: true,
                documentRangeFormattingProvider: true,
            };
        },
        enumerable: true,
        configurable: true
    });
    LightweightLanguageClient.prototype.subscribe = function (notificationName, evt) {
        this._subscriptions[notificationName] = evt;
    };
    LightweightLanguageClient.prototype.sendRequest = function (fileName, requestName, protocolArguments) {
        return __awaiter(this, void 0, void 0, function () {
            var handler, unwrappedValue;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        handler = this._requestHandler[requestName];
                        return [4 /*yield*/, this._unwrapThunk(protocolArguments)];
                    case 1:
                        unwrappedValue = _a.sent();
                        if (handler) {
                            return [2 /*return*/, handler(requestName, unwrappedValue)];
                        }
                        else {
                            return [2 /*return*/, Promise.reject("Not implemented")];
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    LightweightLanguageClient.prototype.sendNotification = function (fileName, notificationName, protocolArguments) {
        return __awaiter(this, void 0, void 0, function () {
            var notifier, unwrappedValue;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        notifier = this._notificationHandler[notificationName];
                        return [4 /*yield*/, this._unwrapThunk(protocolArguments)];
                    case 1:
                        unwrappedValue = _a.sent();
                        if (notifier) {
                            notifier(notificationName, unwrappedValue);
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    LightweightLanguageClient.prototype.handleRequest = function (requestName, handler) {
        this._clientRequestHandler[requestName] = handler;
    };
    LightweightLanguageClient.prototype._handleRequest = function (requestName, handler) {
        this._requestHandler[requestName] = handler;
    };
    LightweightLanguageClient.prototype._getClientRequestHandler = function (requestName) {
        return this._clientRequestHandler[requestName];
    };
    LightweightLanguageClient.prototype._handleNotification = function (notificationName, notificationHandler) {
        this._notificationHandler[notificationName] = notificationHandler;
    };
    LightweightLanguageClient.prototype._notify = function (notificationName, language, payload) {
        var notifierEvent = this._subscriptions[notificationName];
        if (notifierEvent) {
            ;
            notifierEvent.dispatch({
                language: language,
                payload: payload,
            });
        }
    };
    LightweightLanguageClient.prototype._unwrapThunk = function (valueOrThunk) {
        if (typeof valueOrThunk !== "function") {
            return valueOrThunk;
        }
        else {
            var val = valueOrThunk(this.serverCapabilities);
            if (!val) {
                return Promise.resolve(val);
            }
            else if (typeof val.then === "function") {
                return val;
            }
            else {
                return Promise.resolve(val);
            }
        }
    };
    return LightweightLanguageClient;
}());
exports.LightweightLanguageClient = LightweightLanguageClient;
//# sourceMappingURL=LightweightLanguageClient.js.map