"use strict";
/**
 * Utility.ts
 */
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var types = require("vscode-languageserver-types");
exports.zeroBasedPositionToOneBasedPosition = function (zeroBasedPosition) { return ({
    line: zeroBasedPosition.line + 1,
    character: zeroBasedPosition.character + 1,
}); };
exports.getLanguageFromFileName = function (fileName) {
    var extension = path.extname(fileName);
    var language = extension === ".js" || extension === ".jsx" ? "javascript" : "typescript";
    return language;
};
exports.convertCodeEditToTextEdit = function (codeEdit) {
    var range = types.Range.create(codeEdit.start.line - 1, codeEdit.start.offset - 1, codeEdit.end.line - 1, codeEdit.end.offset - 1);
    var newText = codeEdit.newText;
    return {
        range: range,
        newText: newText,
    };
};
exports.convertToDisplayString = function (displayParts) {
    var ret = "";
    if (!displayParts || !displayParts.forEach) {
        return ret;
    }
    displayParts.forEach(function (dp) {
        ret += dp.text;
    });
    return ret;
};
exports.convertTypeScriptKindToSymbolKind = function (kind) {
    switch (kind) {
        case "var":
        case "let":
            return types.SymbolKind.Variable;
        case "property":
        case "getter":
            return types.SymbolKind.Property;
        case "const":
            return types.SymbolKind.Constant;
        case "method":
            return types.SymbolKind.Method;
        case "interface":
            return types.SymbolKind.Interface;
        case "type":
            return types.SymbolKind.Constructor;
        case "class":
            return types.SymbolKind.Class;
        case "module":
            return types.SymbolKind.Module;
        case "alias":
            return types.SymbolKind.Variable;
        case "function":
            return types.SymbolKind.Function;
        case "enum member":
            return types.SymbolKind.Enum;
        default:
            return types.SymbolKind.Field;
    }
};
exports.convertTextSpanToRange = function (span) {
    return types.Range.create(span.start.line - 1, span.start.offset - 1, span.end.line - 1, span.end.offset - 1);
};
exports.convertTextSpanToLocation = function (fileUri, span) {
    var range = exports.convertTextSpanToRange(span);
    return types.Location.create(fileUri, range);
};
//# sourceMappingURL=Utility.js.map