"use strict";
/**
 * index.ts
 *
 * Entry point for ONI's TypeScript Language Service integraiton
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var types = require("vscode-languageserver-types");
var CodeActions_1 = require("./CodeActions");
var Completion_1 = require("./Completion");
var Definition_1 = require("./Definition");
var FindAllReferences_1 = require("./FindAllReferences");
var Formatting_1 = require("./Formatting");
var LightweightLanguageClient_1 = require("./LightweightLanguageClient");
var QuickInfo_1 = require("./QuickInfo");
var Rename_1 = require("./Rename");
var SignatureHelp_1 = require("./SignatureHelp");
var Symbols_1 = require("./Symbols");
var TypeScriptConfigurationEditor_1 = require("./TypeScriptConfigurationEditor");
var TypeScriptServerHost_1 = require("./TypeScriptServerHost");
var Utility = require("./Utility");
exports.activate = function (oni) {
    var _host = null;
    var anyConfig = oni.configuration;
    var anyProcess = oni.process;
    // Add typescript as an editor
    anyConfig.registerEditor("typescript", new TypeScriptConfigurationEditor_1.TypeScriptConfigurationEditor(anyProcess.getDirname()));
    var initializeHost = function (host) {
        host.on("semanticDiag", function (diagnostics) {
            var fileName = diagnostics.file;
            var diags = diagnostics.diagnostics || [];
            var errors = diags.map(function (d) {
                // Convert lines to zero-based to accomodate protocol
                var startPosition = types.Position.create(d.start.line - 1, d.start.offset - 1);
                var endPosition = types.Position.create(d.end.line - 1, d.end.offset - 1);
                var range = types.Range.create(startPosition, endPosition);
                var ret = {
                    // type: null,
                    code: d.code,
                    message: d.text,
                    range: range,
                    severity: types.DiagnosticSeverity.Error,
                };
                return ret;
            });
            var language = Utility.getLanguageFromFileName(fileName);
            connection.notify("textDocument/publishDiagnostics", language, {
                uri: oni.language.wrapPathInFileUri(fileName),
                diagnostics: errors,
            });
        });
        connection.subscribeToRequest("completionItem/resolve", Completion_1.getCompletionDetails(host));
        connection.subscribeToNotification("textDocument/didChange", protocolChangeFile(host));
        connection.subscribeToNotification("textDocument/didSave", onSaved(host));
        connection.subscribeToRequest("textDocument/completion", Completion_1.getCompletions(oni, host));
        connection.subscribeToRequest("textDocument/codeAction", CodeActions_1.getCodeActions(oni, host));
        connection.subscribeToRequest("textDocument/definition", Definition_1.getDefinition(oni, host));
        connection.subscribeToRequest("textDocument/hover", QuickInfo_1.getQuickInfo(oni, host));
        connection.subscribeToRequest("textDocument/rangeFormatting", Formatting_1.formatRange(oni, host));
        connection.subscribeToRequest("textDocument/references", FindAllReferences_1.findAllReferences(oni, host));
        connection.subscribeToRequest("textDocument/rename", Rename_1.doRename(oni, host));
        connection.subscribeToRequest("textDocument/signatureHelp", SignatureHelp_1.getSignatureHelp(oni, host));
        connection.subscribeToRequest("textDocument/documentSymbol", Symbols_1.getDocumentSymbols(oni, host));
        connection.subscribeToRequest("workspace/executeCommand", CodeActions_1.executeCommand(connection, oni, host));
        connection.subscribeToRequest("workspace/symbol", Symbols_1.getWorkspaceSymbols(oni, host));
    };
    var getHost = function () {
        if (!_host) {
            _host = new TypeScriptServerHost_1.TypeScriptServerHost(oni);
            initializeHost(_host);
        }
        return _host;
    };
    var _lightweightLanguageClient = new LightweightLanguageClient_1.LightweightLanguageClient();
    var jsLspCommand = oni.configuration.getValue("language.javascript.languageServer.command");
    var jsLspArgs = oni.configuration.getValue("language.javascript.languageServer.arguments");
    var tsLspCommand = oni.configuration.getValue("language.typescript.languageServer.command");
    var tsLspArgs = oni.configuration.getValue("language.typescript.languageServer.arguments");
    var existingJsLSP = !!(jsLspCommand || jsLspArgs);
    var existingTsLSP = !!(tsLspCommand || tsLspArgs);
    if (!existingJsLSP) {
        oni.language.registerLanguageClient("javascript", _lightweightLanguageClient);
    }
    if (!existingTsLSP) {
        oni.language.registerLanguageClient("typescript", _lightweightLanguageClient);
    }
    var connection = new LightweightLanguageClient_1.LanguageConnection(_lightweightLanguageClient);
    // Subscribe to textDocument/didOpen initially, to kick off
    // initialization of the language server
    var protocolOpenFile = function (message, payload) {
        var textDocument = payload.textDocument;
        var filePath = oni.language.unwrapFileUriPath(textDocument.uri);
        getHost().openFile(filePath, textDocument.text);
    };
    connection.subscribeToNotification("textDocument/didOpen", protocolOpenFile);
    var isSingleLineChange = function (range) {
        if (range.start.line === range.end.line) {
            return true;
        }
        if (range.start.character === 0 &&
            range.end.character === 0 &&
            range.start.line + 1 === range.end.line) {
            return true;
        }
        return false;
    };
    var removeNewLines = function (str) {
        return str.replace(/(\r\n|\n|\r)/gm, "");
    };
    var protocolChangeFile = function (host) { return function (message, payload) { return __awaiter(_this, void 0, void 0, function () {
        var textDocument, contentChanges, filePath, change, saveFile;
        return __generator(this, function (_a) {
            textDocument = payload.textDocument;
            contentChanges = payload.contentChanges;
            if (!contentChanges || !contentChanges.length) {
                return [2 /*return*/];
            }
            if (contentChanges.length > 1) {
                oni.log.warn("Only handling first content change");
            }
            filePath = oni.language.unwrapFileUriPath(textDocument.uri);
            change = contentChanges[0];
            if (!change.range) {
                host.updateFile(filePath, change.text);
            }
            else if (isSingleLineChange(change.range) && change.text) {
                host.changeLineInFile(filePath, change.range.start.line + 1, removeNewLines(change.text));
            }
            else {
                oni.log.warn("Unhandled change request!");
            }
            saveFile = oni.configuration.getValue("debug.typescript.saveFile");
            if (saveFile) {
                host.saveTo(filePath, saveFile);
            }
            // Update errors for modified file
            host.getErrors(filePath);
            return [2 /*return*/];
        });
    }); }; };
    var onSaved = function (host) { return function (protocolName, payload) {
        var textDocument = payload.textDocument;
        var filePath = oni.language.unwrapFileUriPath(textDocument.uri);
        host.getErrorsAcrossProject(filePath);
    }; };
};
//# sourceMappingURL=index.js.map